;; Cyborg Chicken Fight
;;
;; Copyright 2017 - By Michael Kohn
;; http://www.mikekohn.net/
;; mike@mikekohn.net
;;
;; Update to the RC Food Project this time with chicken bones and
;; fried chicken as the car with a chicken bone weapon so they can
;; fight.

.msp430
.include "msp430x2xx.inc"

.define CONTROLLER_0

;  r4 = state (0=idle, 1=header_on, 2=header_off, 3=first half, 4=second)
;  r5 = interupt count
;  r6 =
;  r7 = current byte
;  r8 = bit count
;  r9 = controller bits
; r10 =
; r11 =
; r12 = servo duty cycle interrupt count
; r13 = servo down interrupt count
; r14 = fire button count
; r15 = temp in main

; Command byte is
; SScccCCC
; C = 0: stop moving
; C = 1: forward
; C = 2: down
; C = 3: left
; C = 4; right
; C = 5; fire
; ccc is a repeat of CCC
; SS = 01: controller 1
; SS = 10: controller 2

; 2.0 ms = 152 interrupts
; 0.3 ms = 23 interrupts
; 0.6 ms = 46 interrupts

; 500ms = 38000 interrupts
;   1ms =    76 interrupts
;   1ms =   152 interrupts
;  20ms =  1520 interrupts
; 200ms = 15200 interrupts

RAM equ 0x0200
SERVO_LEFT equ RAM+2
SERVO_RIGHT equ RAM+4
SERVO_LEFT_NEW equ RAM+6
SERVO_RIGHT_NEW equ RAM+8
WATCHDOG equ RAM+16

DOWN_COUNT equ 10
LEFT_ARM_LOWERED equ 76
LEFT_ARM_RAISED equ 152
RIGHT_ARM_LOWERED equ 152
RIGHT_ARM_RAISED equ 76

HEADER equ 152
SHORT equ 23
LONG equ 46

INTERRUPT_MAX equ 5000
WATCHDOG_RESET equ 20000

.org 0xf800
start:
  ;; Turn off watchdog
  ;mov.w #(WDTPW|WDTHOLD), &WDTCTL

  ;; Set watchdog to clock / 32768.
  mov.w #(WDTPW|WDTIS0), &WDTCTL

  ;; Turn off interrupts
  dint

  ;; Set up stack pointer
  mov.w #0x0280, SP

  ;; Set MCLK to 16 MHz with DCO 
  mov.b #DCO_4, &DCOCTL
  mov.b #RSEL_15, &BCSCTL1
  mov.b #0, &BCSCTL2

  ;; Set up output pins
  ;; P1.0 = IR Input
  ;; P1.1 = Debug LED
  ;; P1.2 = Servo Motor Left
  ;; P1.3 = Servo Motor Right
  ;; P1.4 = Motor Left
  ;; P1.5 = Motor Left
  ;; P1.6 = Motor Right
  ;; P1.7 = Motor Right
  ;; P2.6 = Controller Select
  mov.b #0xfe, &P1DIR
  mov.b #0x00, &P1OUT
  mov.b #0x00, &P2DIR
  mov.b #0x40, &P2OUT
  mov.b #0x40, &P2REN
  mov.b #0x00, &P2SEL
  mov.b #0x00, &P2SEL2

  ;; Set up Timer
  mov.w #210, &TACCR0
  mov.w #(TASSEL_2|MC_1), &TACTL ; SMCLK, DIV1, COUNT to TACCR0
  mov.w #CCIE, &TACCTL0
  mov.w #0, &TACCTL1

  mov.w #LEFT_ARM_RAISED, &SERVO_LEFT
  mov.w #RIGHT_ARM_RAISED, &SERVO_RIGHT
  mov.w #0, r12
  mov.w #0, r13

  ;; Change controller number parameters based on jumper.
  bit.b #0x40, &P2IN
  jnz set_controller_1
  mov.b #0x40, r9
  jmp set_controller_done
set_controller_1:
  mov.b #0x80, r9
set_controller_done:

  ;; Enable interrupts
  eint

main:
  ;; Kick the dog
  mov.w #(WDTPW|WDTCNTCL), &WDTCTL

  ;; Debug LED Off
  bis.b #0x02, &P1OUT
  bit.b #0x01, &P1IN
  jeq read_command
  jmp main

read_command:
  ;; Debug LED On
  bic.b #0x02, &P1OUT

  mov.w #0, r5
wait_low:
  ;; Kick the dog
  mov.w #(WDTPW|WDTCNTCL), &WDTCTL
  bit.b #0x01, &P1IN
  jz wait_low

  ;; if r5 is less than 100, something went wrong.  Not sure how important
  ;; this check really is
  cmp.w #100, r5
  jlo main

  ;; Reset watchdog
  mov.w #WATCHDOG_RESET, &WATCHDOG

start_signal:
  ;; signal should be low, start counting?
  ;; check there is no IR
  mov.w #0, r5
wait_ir_off_header:
  ;; Kick the dog
  mov.w #(WDTPW|WDTCNTCL), &WDTCTL
  cmp.w #HEADER+50, r5
  jhs main                ; pause is wayyyy too long, bail out
  bit.b #0x01, &P1IN
  jnz wait_ir_off_header  ; wait for data on IR sensor

receive_next_byte:
  mov.b #0, r7            ; shouldn't be needed
  mov.b #8, r8
receive_next_bit:
  ;; Kick the dog
  mov.w #(WDTPW|WDTCNTCL), &WDTCTL

  ;; check IR signal is on
  mov.w #0, r5
wait_ir_on:
  cmp.w #LONG, r5
  jhs main                ; signal is wayyyy too long, bail out
  bit.b #0x01, &P1IN
  jz wait_ir_on           ; wait for data on IR sensor

  ;; check there is no IR
  mov.w #0, r5
wait_ir_off:
  cmp.w #LONG*2, r5
  jhs main                ; pause is wayyyy too long, bail out
  bit.b #0x01, &P1IN
  jnz wait_ir_off         ; wait for data on IR sensor

  rla.b r7
  cmp #SHORT+12, r5
  jlo not_a_zero
  bis.b #1, r7
not_a_zero:

  dec.b r8
  jnz receive_next_bit

  ;; r7 should now have command (1 byte) from IR
  ;; First check if the correct controller bit is set
  bit.b r9, r7 ; controller number
  jz main

  ;; Kick the dog
  mov.w #(WDTPW|WDTCNTCL), &WDTCTL

  ;; Mask non-command bits and check to make sure CCC and ccc are the
  ;; same.  Otherwise the transmission has errors so need to ignore it.
  mov.b r7, r15
  and.b #0x07, r7 
  and.b #0x38, r15
  rra.b r15
  rra.b r15
  rra.b r15
  cmp.b r7, r15
  jnz main

  cmp.b #1, r7
  jnz not_forward
  dint
  mov.b &P1OUT, r7
  bic.b #0xf0, r7
  bis.b #0xa0, r7
  mov.b r7, &P1OUT
  eint
  jmp main
not_forward:
  cmp.b #2, r7
  jnz not_down
  dint
  mov.b &P1OUT, r7
  bic.b #0xf0, r7
  bis.b #0x50, r7
  mov.b r7, &P1OUT
  eint
  jmp main
not_down:
  cmp.b #3, r7
  jnz not_left
  dint
  mov.b &P1OUT, r7
  bic.b #0xf0, r7
  bis.b #0x60, r7
  mov.b r7, &P1OUT
  eint
  jmp main
not_left:
  cmp.b #4, r7
  jnz not_right
  dint
  mov.b &P1OUT, r7
  bic.b #0xf0, r7
  bis.b #0x90, r7
  mov.b r7, &P1OUT
  eint
  jmp main
not_right:
  cmp.b #5, r7
  jnz not_fire
  cmp.b #0, r13
  jnz not_fire
  bic.b #0xf0, &P1OUT
  mov.w #DOWN_COUNT, r13
  inc.w r14
  bit.w #1, r14
  jnz update_right
  mov.w #LEFT_ARM_LOWERED, &SERVO_LEFT_NEW
  jmp main
update_right:
  mov.w #RIGHT_ARM_LOWERED, &SERVO_RIGHT_NEW
  jmp main
not_fire:
  bic.b #0xf0, &P1OUT
  jmp main

timer_interrupt:
  inc.w r5

  dec.w &WATCHDOG
  jnz watchdog_okay
  bic.b #0xf0, &P1OUT
watchdog_okay:

  inc.w r12
  cmp.w #1520, r12
  jnz not_20ms
  mov.w #0, r12
  bic.b #0x0c, &P1OUT
  mov.w &SERVO_LEFT_NEW, &SERVO_LEFT
  mov.w &SERVO_RIGHT_NEW, &SERVO_RIGHT
  cmp.w #0, r13
  jne update_arm_count
  mov.w #LEFT_ARM_RAISED, &SERVO_LEFT_NEW
  mov.w #RIGHT_ARM_RAISED, &SERVO_RIGHT_NEW
  jmp not_20ms
update_arm_count:
  dec.w r13
not_20ms:

  cmp.w &SERVO_LEFT, r12
  jnz no_servo_left_update
  bis.b #0x08, &P1OUT
no_servo_left_update:

  cmp.w &SERVO_RIGHT, r12
  jnz no_servo_right_update
  bis.b #0x04, &P1OUT
no_servo_right_update:

  reti 

  ;; Vectors
.org 0xfff2
  dw timer_interrupt       ; Timer_A2 TACCR0, CCIFG
.org 0xfffc
  dw start                 ; Oscillator fault, flash access violation
.org 0xfffe
  dw start                 ; Reset



